==> open-xchange-server_6.12.0.0.orig.tar.gz: open-xchange-server-6.12.0.0/src/com/openexchange/ajp13/NonBlockingBlocker.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2006 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.ajp13;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * {@link NonBlockingBlocker} - Non-blocking reentrant blocker; also useful to wrap an existing {@link Runnable runnable}.
 * 
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class NonBlockingBlocker implements Blocker, Runnable {

    private volatile Runnable runnable;

    private final AtomicInteger running;

    private final AtomicInteger writeCounter;

    private volatile Thread owner;

    /**
     * Initializes a new {@link NonBlockingBlocker}.
     */
    public NonBlockingBlocker() {
        this(null);
    }

    /**
     * Initializes a new {@link NonBlockingBlocker} wrapping given {@link Runnable runnable}.
     * 
     * @param runnable The runnable to block
     */
    public NonBlockingBlocker(final Runnable runnable) {
        super();
        writeCounter = new AtomicInteger();
        running = new AtomicInteger();
        this.runnable = runnable;
    }

    public void block() {
        synchronized (this) {
            final Thread cur = Thread.currentThread();
            if (cur == owner) {
                // This thread already blocks
                return;
            }
            // Check if another thread still holds the block
            int state = writeCounter.get();
            while ((state & 1) == 1) {
                /*
                 * Blocked access in progress
                 */
                state = writeCounter.get();
            }
            // Set blocked
            writeCounter.getAndIncrement();
            owner = cur;
            // Wait for other threads leaving
            int i = running.get();
            while (i > 0) {
                i = running.get();
            }
        }
    }

    public void unblock() {
        if (owner != null && Thread.currentThread() != owner) {
            throw new IllegalMonitorStateException("Thread " + Thread.currentThread().getName() + " does not own this blocker");
        }
        writeCounter.getAndIncrement();
    }

    public void acquire() {
        if (owner != null && Thread.currentThread() == owner) {
            // Owning thread!
            return;
        }
        int state = writeCounter.get();
        while ((state & 1) == 1) {
            /*
             * Blocked access in progress
             */
            state = writeCounter.get();
        }
        running.incrementAndGet();
    }

    public void release() {
        if (owner != null && Thread.currentThread() == owner) {
            // Owning thread!
            return;
        }
        running.decrementAndGet();
    }

    public void run() {
        acquire();
        try {
            if (runnable != null) {
                runnable.run();
            }
        } finally {
            release();
        }
    }

}

==> open-xchange-server_6.14.0.0.orig.tar.gz: open-xchange-server-6.14.0.0/src/com/openexchange/ajp13/NonBlockingBlocker.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2006 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.ajp13;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * {@link NonBlockingBlocker} - Non-blocking reentrant blocker; also useful to wrap an existing {@link Runnable runnable}.
 * 
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class NonBlockingBlocker implements Blocker, Runnable {

    private volatile Runnable runnable;

    private final Object lock;

    private final AtomicInteger running;

    private final AtomicInteger writeCounter;

    private volatile Thread owner;

    /**
     * Initializes a new {@link NonBlockingBlocker}.
     */
    public NonBlockingBlocker() {
        this(null);
    }

    /**
     * Initializes a new {@link NonBlockingBlocker} wrapping given {@link Runnable runnable}.
     * 
     * @param runnable The runnable to block
     */
    public NonBlockingBlocker(final Runnable runnable) {
        super();
        lock = new Object();
        writeCounter = new AtomicInteger();
        running = new AtomicInteger();
        this.runnable = runnable;
    }

    public void block() {
        synchronized (lock) {
            final Thread cur = Thread.currentThread();
            if (cur == owner) {
                // This thread already blocks
                return;
            }
            while (null != owner) {
                // Another thread already blocks, wait for being unblocked
                try {
                    lock.wait();
                } catch (final InterruptedException e) {
                    cur.interrupt();
                    throw new IllegalMonitorStateException("Thread " + cur.getName() + " was interrupted.");
                }

            }
            // Check if another thread still holds the block
            int state = writeCounter.get();
            while ((state & 1) == 1) {
                /*
                 * Blocked access in progress
                 */
                state = writeCounter.get();
            }
            // Set blocked
            writeCounter.getAndIncrement();
            owner = cur;
            // Wait for other threads leaving
            int i = running.get();
            while (i > 0) {
                i = running.get();
            }
        }
    }

    public void unblock() {
        synchronized (lock) {
            if (null == owner || Thread.currentThread() != owner) {
                throw new IllegalMonitorStateException("Thread " + Thread.currentThread().getName() + " does not own this blocker");
            }
            // Set unblocked
            writeCounter.getAndIncrement();
            owner = null;
            lock.notifyAll();
        }
    }

    public void acquire() {
        if (owner != null && Thread.currentThread() == owner) {
            // Owning thread!
            return;
        }
        int state = writeCounter.get();
        while ((state & 1) == 1) {
            /*
             * Blocked access in progress
             */
            state = writeCounter.get();
        }
        running.incrementAndGet();
    }

    public void release() {
        if (owner != null && Thread.currentThread() == owner) {
            // Owning thread!
            return;
        }
        running.decrementAndGet();
    }

    public void run() {
        acquire();
        try {
            if (runnable != null) {
                runnable.run();
            }
        } finally {
            release();
        }
    }

}
